# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################

# DEPRECATED
# DEPRECATED
# DEPRECATED
# DEPRECATED
# DEPRECATED
# DEPRECATED
# DEPRECATED
# DEPRECATED
# DEPRECATED
# DEPRECATED
# DEPRECATED
# DEPRECATED
# DEPRECATED
# DEPRECATED
# DEPRECATED
# DEPRECATED, should disappear, one day

"""
Contains objects which auto-attach on parent
Theses objects should be in the parent, they are not displayed by default
they have got :

    * init(parent), which auto attach on parent
    * show(...), which show the object
    * close(..), to hide the object

except "Clipboard", which have init(parent) & copy(str)

"""

from .mbox import MBox
from .toaster import Toaster
from .popmenu import PopMenu
from .clipboard import Clipboard



# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################

from htag import Tag

class Clipboard(Tag.div):
    tag="div"
    def __init__(self,parent):
        """ auto attach parent """
        print("**DEPRECATED** don't use Clipboard, use the new Service !")
        super().__init__()
        parent <= self

    def copy(self,txt):
        assert "`" not in txt # ;-)
        self.call("""
let ta = document.createElement('textarea');
ta.value = `%s`;
self.appendChild(ta);
ta.select();
document.execCommand('copy');
self.removeChild(ta);
""" % txt)

if __name__=="__main__":
    from htag.runners import PyWebWiew,BrowserHTTP

    class Obj(Tag.div):
        def __init__(self):
            super().__init__()
            self.cc = Clipboard(self)
            self<= Tag.button( "Copy", _onclick=self.bind.copy())

        def copy(self):
            self.cc.copy("backend text to copy in clipboard")

    BrowserHTTP( Obj() ).run()



# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################
from . import TagBulma
from htag import Tag

class NavSide(TagBulma.div):
    statics = (
        Tag.style(r"""
.aburger {
  transform:scale(1.8,1.3);
  vertical-align: text-bottom;
  padding:3px;
  margin-right:8px;
  cursor:pointer;
}

div.backmenu {
    display:none;
    z-index:9;
    position:fixed;
    top:0px;
    bottom:0px;
    left:0px;
    right:0px;
    background:#888;
    opacity:0.5;
}

div.menupop {
    position:fixed;
    z-index:20;
    top: var(--ns-top);
    bottom:0px;
    left:0px;
    width: var(--ns-width);
    height:100%;
    overflow:auto;
    background:white;

    -webkit-transition: left 0.2s ease-in-out;
    -moz-transition: left 0.2s ease-in-out;
    -ms-transition: left 0.2s ease-in-out;
    -o-transition: left 0.2s ease-in-out;

}

body {
    padding-left: var(--ns-width) !important;
    padding-top: var(--ns-top) !important;
}

.onlyBig   { display:inherit }
.onlySmall { display:none }

@media (max-width: 800px) {

    div.menupop { left: var(--ns-width-mobile-sub) }

    body { padding-left:0px !important }

    .menuShow div.menupop {
        left: 0px;
        width: var( --ns-width-mobile );
    }
    .menuShow div.backmenu { display:block }

    .onlyBig   { display:none }
    .onlySmall { display:inherit }

}"""),
        Tag.script("""
function switchMenu() { document.body.classList.toggle("menuShow") }
function hideMenu()   { document.body.classList.remove("menuShow") }
""")
    )
    def init(self,title,sidecontent,width:str="200px",width_small:str="80%",class_color:str="is-black", **a):

        bb=Tag.div("&#9776;",_class="aburger onlySmall",_onclick="switchMenu()")

        nav= Tag.nav(_role="navigation",_aria_label="main navigation", _class="navbar is-fixed-top "+class_color)
        nav <= Tag.div(_class="navbar-brand") <= Tag.span(bb + title,  _class="navbar-item")

        width_m="80%"

        self <= Tag.style(f"""
:root {{
  --ns-top: 52px;
  --ns-width: {width};
  --ns-width-mobile: {width_small};
  --ns-width-mobile-sub: -{width_small};
}}
        """)
        self <= nav
        self <= Tag.div(_class="backmenu click",_onclick="switchMenu()")    # greyed background
        self <= Tag.div(sidecontent,_class="menupop")

    def hide(self):
        """ hide menu if on mobile """
        self.call("""hideMenu()""")




# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################
from . import TagBulma, Box, Fields, HBox, Form, Content
from htag import Tag
from htag.tag import Caller
import json,html


def warn(o,where):
    print(f"**WARNING** DEPRECATED remove '{where}()' for ",o.__class__.__name__)

from . import inputs2

class OldInputCompat:
    def __init__(self,onchange=None):
        if onchange:
            self.onchange(onchange)
    def onchange(self,cb):
        warn(self,"onchange")
        self["onchange"].bind( cb )
    def setValue(self,value):
        warn(self,"setValue")
        self.value = value

class Input(inputs2.Input,OldInputCompat):
    def __init__(self, value, options:inputs2.ListOrDict=[], name=None, onchange=None,**a):
        super().__init__(value,options,name=name, **a)
        OldInputCompat.__init__(self, onchange)


class Range(inputs2.Range,OldInputCompat):
    def __init__(self, value, name=None,onchange=None,**a):
        super().__init__(value,name=name, **a)
        OldInputCompat.__init__(self, onchange)


class Checkbox(inputs2.Checkbox,OldInputCompat):
    def __init__(self, value:bool, label:str, name=None,onchange=None,**a):
        super().__init__(value,label=label,name=name, **a)
        OldInputCompat.__init__(self, onchange)


class Radio(inputs2.Radio,OldInputCompat):
    def __init__(self, value, options:inputs2.ListOrDict, name=None,onchange=None,**a):
        super().__init__(value,options,name=name, **a)
        OldInputCompat.__init__(self, onchange)


class Select(inputs2.Select,OldInputCompat):
    def __init__(self, value, options:inputs2.ListOrDict, name=None,onchange=None,**a):
        super().__init__(value,options,name=name, **a)
        OldInputCompat.__init__(self, onchange)


class Textarea(inputs2.Textarea,OldInputCompat):
    def __init__(self, value:str, name=None,onchange=None,**a):
        super().__init__(value,name=name, **a)
        OldInputCompat.__init__(self, onchange)


class SelectButtons(inputs2.SelectButtons,OldInputCompat):
    def __init__(self, value, options:inputs2.ListOrDict, name=None,onchange=None,**a):
        super().__init__(value,options,name=name, **a)
        OldInputCompat.__init__(self, onchange)


class TabsHeader(SelectButtons):
    _bstyle_="is-centered" # "is-boxed is-centered"




##############################################################################
##############################################################################
##############################################################################

if __name__=="__main__":

    LIST=[1,2,3]
    DICT = dict(A="Albert",B="Bonnie",C="Clyde")

    class Page(Tag.div):
        statics = [Tag.style(""".myclass {border: 1px solid blue !important;background:#AFA !important}""")]
        def __init__(self):
            super().__init__()

            self.modders = SelectButtons(0,{0:"normal",4:"all required",5:"all readonly", 1:"all disabled",2:"style",3:"class"}, _onchange= self.redraw )

            self.tab = TabsHeader(0,{0:"Via Form",1:"Reactive",2:"Inside Fields"}, _onchange= self.redraw )
            self.visu = Tag.div()
            self.pre = Tag.pre()

            self <= self.modders
            self <= self.tab
            self <= self.visu
            self <= self.pre

            self.redraw( None )

        def redraw(self, notUsed ):
            if self.modders.value ==1:
                commons={"_disabled": True}
            elif self.modders.value ==4:
                commons={"_required": True}
            elif self.modders.value ==5:
                commons={"_readonly": True}
            elif self.modders.value ==2:
                commons={"_style": "border: 1px solid red !important;background:yellow !important"}
            elif self.modders.value ==3:
                commons={"_class": "myclass"}
            else:
                commons={}

            print("COMMONS:",commons)

            if self.tab.value==0:
                #====================================================
                # FORM
                #====================================================
                f=Form(onsubmit=self.formSubmit)
                f<= Content("All inputs are sent thru a 'form' via the submit button below")
                f<=Textarea("hello", name="mytext", **commons)
                f<= HBox(
                    Input(2,name="myinput1",**commons),
                    Input(None,LIST,name="myinput2",**commons),
                    Input(None,DICT,name="myinput3",**commons),
                )
                f<= HBox(
                    Input("",name="date",_type="date",**commons),
                    Input("",name="month",_type="month",**commons),
                    Input("",name="time",_type="time",**commons),
                    Input("",name="pass",_type="password",**commons),
                    Input("",name="color",_type="color",**commons),
                    Input("",name="file",_type="file",**commons), # non sense, in a form
                )

                f<= HBox(
                    Radio(2,LIST,name="myradio1",**commons),
                    Radio("B",DICT,name="myradio2",**commons),
                )

                f<= HBox(
                    SelectButtons(2,LIST,name="mysb1",**commons),
                    SelectButtons("B",DICT,name="mysb2",**commons),
                )

                f<= HBox(
                    Select(2,LIST,name="myselect1",**commons),
                    Select("B",DICT,name="myselect2",**commons),
                )

                f<= HBox(
                    Checkbox(True,"homme",name="mycb1",**commons),
                    Checkbox(False,"femme",name="mycb1",**commons),
                )

                f<= Range(12, name="myrange", _min=10, _max=78, _step=2, **commons)

                f<= Tag.button("ok",_class="button")

                self.pre.set( "^^ Use the SUBMIT BUTTON ^^" )
            elif self.tab.value==1:
                #====================================================
                # Reactive
                #====================================================
                f=Tag.div()
                f<= Content("All inputs are reactive and sent itself to the react() method")
                f<= Textarea("hello",_onchange= self.react,**commons)
                f<= HBox(
                    Input(2,_onchange= self.react,**commons),
                    Input(None,LIST,_onchange= self.react,**commons),
                    Input(None,DICT,_onchange= self.react,**commons),
                )

                f<= HBox(
                    Input("",_type="date",_onchange= self.react, **commons),
                    Input("",_type="month",_onchange= self.react,**commons),
                    Input("",_type="time",_onchange= self.react,**commons),
                    Input("",_type="password",_onchange= self.react,**commons),
                    Input("",_type="color",_onchange= self.react,**commons),
                    Input("",_type="file",_onchange= self.react,**commons),    # not a great sense, either !
                )


                f<= HBox(
                    Radio(2,LIST,_onchange= self.react,**commons),
                    Radio("B",DICT,_onchange= self.react,**commons),
                )

                f<= HBox(
                    SelectButtons(2,LIST, _onchange = self.react,**commons),
                    SelectButtons("B",DICT, _onchange = self.react,**commons),
                )

                f<= HBox(
                    Select(2,LIST,_onchange= self.react,**commons),
                    Select("B",DICT,_onchange= self.react,**commons),
                )

                f<= HBox(
                    Checkbox(True,"homme",_onchange= self.react,**commons),
                    Checkbox(False,"femme",_onchange= self.react,**commons),
                )

                f<= Range(12, _min=10, _max=78, _step=2,_onchange= self.react,**commons)

                self.pre.set( "Interact with an input ;-)" )
            else:
                #====================================================
                # just a fieldset/form (without interaction/submit)
                #====================================================
                f=Fields()
                f.addField("Name", Input("mamamam",**commons))
                f.addField("Surname", Input("kookokko",**commons), "not me ;-)")
                f.addField("Sex", Radio(1,LIST,**commons))
                f.addField("Other", [Select(1,DICT,**commons), Select(None,[None,1,2,3],**commons) ])
                f.addField("Address", Textarea("",_rows="2",**commons))
                f.addField("Size", Range(10,_min=0, _max=100, **commons))
                f.addField("Ok with that", Checkbox(False,"Sure?", **commons))

                self.pre.set( "Just a presentation thing ! (no interactions)" )

            #######################################################
            self.visu.set( Box(f) )

        def formSubmit(self, f:dict):
            self.pre.set( json.dumps(f,indent=4) )

        def react(self, obj ):
            self.pre.set( "class:%s id=%s, value = %s (type:%s)" %( obj.__class__.__name__, id(obj), obj.value, html.escape(str(type(obj.value))) ) )



    app=Page()



    from . import _test
    _test( app )



# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################

from htag import Tag
import os,base64


class FileUpload(Tag.input):
    statics=[Tag.script(r"""
function fileupload( self, cbInteract ) {
    let file = self.files[0];
    var reader = new FileReader();
    reader.onload =  e => { cbInteract(file.name, btoa(e.target.result)) };
    reader.readAsBinaryString(file); //reader.readAsText(file);
}""")]

    def __init__(self,onchange,**a):
        Tag.__init__(self,**a)
        self["type"]="file"
        self["onchange"]="fileupload(this, function(name,content) {%s})" % self.bind._onchange( b"name", b"content")
        self.onchange=onchange

    def _onchange(self,name:str,content:str):
        self.onchange(name,base64.b64decode(content))

if __name__=="__main__":
    def showFile(name: str,content: bytes):
        print(name,content)

    obj=FileUpload( showFile )

    from . import _test
    _test( "TRANSFERT AS BINARY STRING",obj )



# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################
from . import TagBulma, A
from htag import Tag

class Nav(TagBulma):
    tag="nav"

    imports = A

    def __init__(self, title, **a):
        super().__init__()
        self["role"]="navigation"
        self["aria-label"]="main navigation"
        self["class"].add("navbar","is-fixed-top","is-black")
        self.title = title
        self._entries_first = {}
        self._entries_end = {}

        self.update()

    def addEntry(self,name,callback,atEnd=False):
        if atEnd:
            self._entries_end[name]=callback
        else:
            self._entries_first[name]=callback
        self.update()

    def update(self):  # DYNAMIC RENDERING HERE !
        self.clear()
        divBrand = Tag.div(_class="navbar-brand")
        divBrand <= Tag.b(self.title,  _class="navbar-item")
        if self._entries_first or self._entries_end:
            divBrand <= A(
                [Tag.span(_aria_hidden=True),
                Tag.span(_aria_hidden=True),
                Tag.span(_aria_hidden=True),
                ],
                _role="button",
                _class="navbar-burger burger",
                _aria_label="menu",
                _aria_expanded="false",
                _data_target="navbarBasicExample",
                _onclick="this.classList.toggle('is-active');document.querySelector('.navbar-menu').classList.toggle('is-active')",
            )

        menu = Tag.div(_class="navbar-start")
        for k, v in self._entries_first.items():
            menu <= A(k, _class="navbar-item", _onclick=self.bind.evtSelectEntry(k))

        if self._entries_end:
            menuEnd = Tag.div(_class="navbar-end")
            for k, v in self._entries_end.items():
                menuEnd <= A(k, _class="navbar-item", _onclick=self.bind.evtSelectEntry(k))
        else:
            menuEnd = None

        divMenu = Tag.div( [menu, menuEnd], _class="navbar-menu")

        self <= divBrand
        self <= divMenu

    def evtSelectEntry(self, name):
        self.call("document.querySelector('.navbar-menu').classList.remove('is-active')")
        self.call("document.querySelector('.navbar-burger').classList.remove('is-active')")
        entries = {**self._entries_first,**self._entries_end}
        callback = entries[name]
        callback()

if __name__=="__main__":

    def nothing():
        print("do nothing")

    def add():
        nav.addEntry( "added page%s" %len(nav._entries_first), nothing )

    nav= Nav("Nav Demo")
    nav.addEntry( "Add a Page ", add )
    nav.addEntry( "exit", nothing, True )


    from . import _test
    _test( nav )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['htbulma', 'htbulma.services']

package_data = \
{'': ['*']}

install_requires = \
['htag>=0.9.20']

setup_kwargs = {
    'name': 'htbulma',
    'version': '0.9.0',
    'description': 'GUI toolkit for creating beautiful applications for mobile, web, and desktop from a single python3 codebase',
    'long_description': '## htbulma = "HTag bulma"\n\nExample of gui toolkit using [htag](https://github.com/manatlan/htag).\n\n<a href="https://pypi.org/project/htbulma/">\n    <img src="https://badge.fury.io/py/htbulma.svg" alt="Package version">\n</a>\n\n**Just to be clear** : This module is just for me ;-). It\'s just here to demonstrate that you can build a py3 module, containnig a set of [htag](https://github.com/manatlan/htag)\'s components, ready to be used, in you [htag](https://github.com/manatlan/htag) app.\n\nOf course, you can use it, at your own risk. But be aware that :\n\n * the current components may change completly (and broke your apps)\n * many of them are not well coded (80% coming from the old gtag, and 70% are not using the last htag features in the right place)\n \nTheses are my components, for my quick\'s htag apps ;-), for demo\'ing the htag goal !\n\nI\'d love too see a real module, well maintained, with high-end and clever components (but I have no time to manage that, focusing on other projects (ex: htag))\n\n[See an example](test.py)\n',
    'author': 'manatlan',
    'author_email': 'manatlan@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/manatlan/htbulma',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################

from htag import Tag
import os
import fnmatch


class FileSelect(Tag.div):
    statics="""
.FileSelect {
    white-space: nowrap;
}
.FileSelect div:hover {
    background: #eee;cursor:pointer;
}
.FileSelect div.selected {
    background:#DDD;
}
""",b"""

function FileSelect_select(id) {
    document.querySelectorAll('div.FileSelect div').forEach(div => {
      div.classList.remove('selected');
    });
    document.getElementById(id).classList.add('selected');
}

"""
    def init(self,path,onselect,pattern="*",**a):
        self["class"].add("FileSelect")
        self.onselect=onselect

        self._patterns = [pattern] if isinstance(pattern,str) else pattern
        self._root = os.path.realpath(path)
        self._selected = None
        self._render(path)

    def refresh(self):
        self._render(self.path)

    def _render(self,path):
        self.clear()
        self.path = os.path.realpath(path)
        assert self.path.startswith(self._root) # security
        folders=[]
        files=[]
        for i in os.listdir(self.path):
            if os.path.isdir( os.path.join(self.path,i) ):
                folders.append(i)
            else:
                files.append(i)

        if self.path != self._root:
            self<= Tag.div( Tag.b( "&#11013; "+ path[len(self._root):]),path=".." , _onclick=self._selectFolder)
        for i in sorted(folders):
            self<= Tag.div( "📁 "+i, path=i, _onclick=self._selectFolder)
        for i in sorted(files):
            if any( [fnmatch.fnmatch(i, p) for p in self._patterns] ):
                self<= Tag.div(i, path=i, _onclick=self._selectFile, _class="selected" if i==self._selected else "")

    def _selectFolder(self,o):
        self._render( os.path.realpath(os.path.join(self.path,o.path)) )
    def _selectFile(self,o):
        path = os.path.realpath(os.path.join(self.path,o.path))
        assert path.startswith(self._root)
        self._selected = o.path
        self.call(f"""FileSelect_select('{id(o)}')""")
        self.onselect(path)

if __name__=="__main__":
    def showFile(name):
        print(name)

    obj=FileSelect( r".", showFile )

    from . import _test
    _test( obj )



# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################

from htag import Tag
from .. import TagBulma,Button


class PopMenu(TagBulma):
    tag="span"

    def __init__(self, parent):
        """ auto attach on 'parent' """
        TagBulma.__init__(self)
        print("**DEPRECATED** don't use PopMenu, use the new Service !")

        parent <= self
        self._parent = parent
        self._menu={}

    def show(self,x:int,y:int,entries:dict):
        self._menu=dict(
            pos=dict(x=x,y=y),
            entries=entries,
        )

    def __str__(self):
        self.clear()
        if self._menu and self._menu.get("entries"):
            entries=Tag.ul(_class="menu-list")
            for name,callback in self._menu["entries"].items():
                entries<= Tag.li( Tag.a(name,_onclick=self.bind.close(name)) )

            if "pos" in self._menu:
                fix="left:%spx;top:%spx" % (self._menu["pos"]["x"],self._menu["pos"]["y"])
            else:
                fix=""

            self <= Tag.div(
                _class="modal-background",
                _onclick=self.bind.close(),
                _style="background-color:inherit"
            )
            self <= Tag.div(
                Tag.aside( entries,_class="menu"),
                _class="card",
                _style="position:fixed;z-index:10000;padding:2px;"+fix,
            )

        return TagBulma.__str__(self)

    def close(self,name=None):
        if name:
            self._menu["entries"][name]()

        self._menu=None

    def binder(self, callbackName: str) -> str:
        """ bind to a parent method called 'callbackName', which should be callbackName(self,x,y) """
        return getattr(self._parent.bind, callbackName)(b"event.clientX",b"event.clientY")


if __name__=="__main__":
    class Obj(Tag.div):
        def __init__(self,**a):
            Tag.__init__(self,**a)

            self.pop = PopMenu(self)

            self <= Button("yo", _onclick=self.pop.binder("aff") )  #TODO: CAN DO BETTER HERE

        def aff(self,x,y):
            entries=dict(
                menu1= lambda: self.nothing(1),
                menu2= lambda: self.nothing(2),
            )
            self.pop.show(x,y, entries)

        def nothing(self,v):
            print("do nothing",v)


    from .. import _test
    _test( Obj() )



# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################

from htag import Tag
from . import TagBulma,Button,HBox,Content


"""
will replace old services MBox,Toaster,PopMenu,Clipboard
with just One (unclear'able) service named 'Service' (poor name)
it does all the same thing, but :
 - in one object Service
 - this instance can reattach automatically (after a clear childs)
 - modal features with keyboard event (esc/return) (except: alert, just escape)
 - all objets are unlimited (ex: you can pop multiple popmenu at a time)
 - all objets use the last highend features of htag (0.8.4 at this time)
 - popmenu can contains a list of object (not only Tag.A), and can open on new event htag feature
 - all are betters

"""

class PopMenu(Tag.div):
    def init(self,entries,x,y):
        omenu=Tag.ul(_class="menu-list")
        for obj in entries:
            if isinstance(obj,Tag) and obj.tag in ["a","A"] and obj["onclick"]:
                # Only a[@onclick] auto-close the popmenu...
                obj["onclick"].bind(self.close)
            omenu += Tag.li( obj )

        self += Tag.div(
            _class="modal-background",
            _onclick=self.close,
            _oncontextmenu = self.bind( self.close ) + "return false",
            _style="background-color:inherit;z-index:1000000"
        )

        js="""(function(tag,x,y) {
            tag.style="position:fixed;z-index:1000001;padding:2px;left:"+x+"px;top:"+y+"px";
            let bw=document.body.clientWidth;
            let bh=document.body.clientHeight;
            let w=tag.clientWidth;
            let h=tag.clientHeight;

            if(x+w > bw) x=bw-w;
            if(y+h > bh) y=bh-h;

            tag.style="position:fixed;z-index:1000001;padding:2px;left:"+x+"px;top:"+y+"px";
        })(self,%s,%s)"""

        self += Tag.div(
            Tag.aside( omenu,_class="menu"),
            _class="card",
            js=js % (x,y),
        )

    def close(self,o=None):
        self.remove()

class Modal(Tag.div):
    def init(self, content, canClose=True, full=False):
        self["class"] = "modal is-active"

        bg=Tag.div(_class="modal-background")
        self += bg
        if canClose:
            bg["onclick"]=self.close
            self += Tag.div(
                _class="modal-close is-large",
                _aria_label="close",
                _onclick=self.close,
            )
            self["onkeydown"]="""if(event.which == 13) %s; if(event.which == 27) %s;""" % (self.bind.onkey(True),self.bind.onkey(False))

        self += Tag.div(
            Tag.div(
                content,
                _tabindex=0,
                js="self.focus()",
                _style="outline: none" + ("height:100%;overflow-y:auto" if full else ""),
                _class="box",
            ),
            _class="modal-content",
            _style = "width:90%;height:98%;" if full else None,
        )

    def onkey(self,valid):
        #default behaviour in modal.mbox
        if valid is False:
            self.close(self)

    def close(self,o=None):
        self.remove()


class Toast(Tag.div):

    def init(self,content):
        self["class"]="notification has-text-light has-background-grey"
        self["style"] = "position:fixed;left:0px;right:0px;bottom:0px;z-index:1000"
        self+= Tag.button(_class="delete", _onclick=self.close)
        self+= content

    def close(self,o=None):
        self.remove()

class Service(TagBulma):

    def init(self,parent):
        self["info"]="htbulma services"
        self._root = parent
        self._reroot()

    def _reroot(self):
        """ ensure that this object is attached to a parent/main (_root)
            if not (after a clear()), it re-adds itself to _root (parented object) ;-)
        """
        if self not in self._root.childs:
            self._root.add(self,True) # force reparent (when Tag.STRICT_MODE)


    def clipboard(self,txt):
        self._reroot()

        assert "`" not in txt # ;-)
        self.call("""
let ta = document.createElement('textarea');
ta.value = `%s`;
self.appendChild(ta);
ta.select();
document.execCommand('copy');
self.removeChild(ta);
""" % txt)


    def popmenu(self,entries:list,pos):
        self._reroot()

        if isinstance(pos,Tag):
            x,y=pos.event["clientX"],pos.event["clientY"]
        elif isinstance(pos,tuple):
            x,y=pos
        else:
            raise Exception("popemenu bad call")

        x=PopMenu(entries,x,y)
        self += x
        return x

    def toast(self,content,delay=2000):
        self._reroot()

        x=Toast(content)
        self += x
        # prefer the self.call() way, to send the js for this case
        # because with x.js, js is re-executed at each redraw
        # and can cause dead objects (when event reach server)
        self.call("""setTimeout(function(){%s},%s);""" % (x.bind.close(None),delay))
        return x

    def alert(self, content, canClose=True, full=False):
        """ "same" signature as js window.alert() """
        self._reroot()

        x=Modal(content, canClose=canClose, full=full)
        self += x
        return x

    def confirm(self, content, ok, ko=None,txtok="OK",txtko="Cancel"): # -> o.value
        """ "same" signature as js window.confirm() """
        self._reroot()

        bko=Button(txtko, _class="is-light", _style="flex: 1 0 25%;")
        bok=Button(txtok, _style="flex: 1 0 25%;")

        main = Content( content )
        main += HBox(
            Tag.H.div(_style="flex: 1 0 25%;"),
            Tag.H.div(_style="flex: 1 0 25%;"),
            bko,
            bok,
        )

        if ko is None:
            ko=lambda o: None

        x=Modal(main, canClose=True)
        x.onkey = lambda valid: x( f"document.getElementById('{id(valid and bok or bko)}').click()" )
        bok["onclick"].bind(ok).bind(x.close)
        bko["onclick"].bind(ko).bind(x.close)

        self += x
        return x


    def prompt(self, title, defaultValue, ok, ko=None,txtok="OK",txtko="Cancel"): # -> o.value
        """ "same" signature as js window.prompt() """
        self._reroot()

        input = Tag.input(_value=defaultValue, js="self.focus();self.setSelectionRange(0, self.value.length)", _class="input")

        bko=Button(txtko, _class="is-light", _style="flex: 1 0 25%;")
        bok=Button(txtok, _style="flex: 1 0 25%;")

        main = Content( Tag.h3(title) )
        main += input
        main += HBox(
            Tag.H.div(_style="flex: 1 0 25%;"),
            Tag.H.div(_style="flex: 1 0 25%;"),
            bko,
            bok,
            _style="margin-top:10px"
        )

        def getv(o,x):
            # save the value in the button object
            o.value = x

        x=Modal(main, canClose=True, full=False)
        x.onkey = lambda valid: x( f"document.getElementById('{id(valid and bok or bko)}').click()" )

        js_get_value = b"document.getElementById('%d').value" % id(input)
        bok["onclick"].bind( getv, js_get_value ).bind(ok).bind(x.close)
        bko["onclick"].bind( getv, js_get_value ).bind(ko).bind(x.close)

        self += x
        return x




# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################
from . import TagBulma, Button
from htag import Tag
import math


class Table(TagBulma):
    """ adapted as is, from gtag.gtags.Table
        NOT OPTIMIZED for htag ... but works as is ;-)
    """
    tag="div"

    def __init__(
        self, rows: list, cols: list = None, pageSize: int = None, pageIndex: int = 0, **a
    ):
        super().__init__(**a)
        self.rows = rows
        self.cols = cols
        self.pageSize = pageSize
        self.pageIndex = pageIndex
        self.update()

    def update(self):
        self.clear()
        if self.cols:
            h = Tag.thead( [Tag.th(col) for col in self.cols] )
        else:
            h = None

        if self.pageSize:
            rows = self.rows[
                self.pageIndex * self.pageSize : self.pageIndex * self.pageSize
                + self.pageSize
            ]
            nbPage = math.ceil(len(self.rows) / self.pageSize)
        else:
            rows = self.rows

        ll = []
        for row in rows:
            row = row if hasattr(row, "__iter__") else [row]
            ll.append(Tag.tr( [Tag.td(col) for col in row]) )

        t=Tag.table( _class="table is-bordered is-striped is-narrow is-hoverable is-fullwidth")
        t <= h
        t <= Tag.tbody(ll)

        self <= t

        if self.pageSize and nbPage >1:
            nav = Tag.nav(
                _class="pagination is-small", _role="navigation", _aria_label="pagination"
            )
            if self.pageIndex > 0:
                nav.add(
                    Tag.a(
                        "<<",
                        _class="pagination-previous",
                        _onclick=self.bind.setPage(self.pageIndex - 1),
                    )
                )
            else:
                nav.add(Tag.a("<<", _class="pagination-previous", _disabled=True))
            if self.pageIndex < nbPage - 1:
                nav.add(
                    Tag.a(
                        ">>",
                        _class="pagination-next",
                        _onclick=self.bind.setPage(self.pageIndex + 1),
                    )
                )
            else:
                nav.add(Tag.a(">>", _class="pagination-next", _disabled=True))

            ul = Tag.ul(_class="pagination-list")
            for i in range(0, nbPage):
                if (i == 0 or i == nbPage - 1 or self.pageIndex - 3 <= i <= self.pageIndex + 3):
                    klass = (
                        "pagination-link is-current"
                        if self.pageIndex == i
                        else "pagination-link"
                    )
                    ul.add(
                        Tag.li(
                            Tag.a(
                                (i + 1),
                                _class=klass,
                                _aria_label="Goto page %s" % (i + 1),
                                _onclick=self.bind.setPage(i),
                            )
                        )
                    )
                    hole = False
                else:
                    if not hole:
                        ul.add(Tag.li(Tag.a("&hellip;", _class="pagination-ellipsis")))
                        hole = True

            nav.add(ul)

            self <= nav

    def setPage(self, p):
        self.pageIndex = p
        self.update()

if __name__=="__main__":

    ll = [( Button(i + 1,_class="is-small"), i + 1, i + 1, i + 1, i + 1) for i in range(33)]
    obj=Table(ll, cols=list("abcde"), pageSize=10, pageIndex=0)

    from . import _test
    _test( obj )
